package com.bergerkiller.bukkit.nolagg;

import org.bukkit.Material;
import org.bukkit.entity.Entity;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockPhysicsEvent;
import org.bukkit.event.entity.CreatureSpawnEvent;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.entity.ItemSpawnEvent;
import org.bukkit.event.entity.CreatureSpawnEvent.SpawnReason;
import org.bukkit.event.world.ChunkLoadEvent;
import org.bukkit.event.world.ChunkUnloadEvent;
import org.bukkit.event.world.WorldLoadEvent;
import org.bukkit.event.world.WorldUnloadEvent;

import com.bergerkiller.bukkit.nolagg.meta.ChunkMetaData;
import com.bergerkiller.bukkit.nolagg.meta.WorldMetaData;
import com.bergerkiller.bukkit.nolagg.monitor.PerformanceMonitor;
import com.bergerkiller.bukkit.nolagg.spawnlimit.EntityLimiter;
import com.bergerkiller.bukkit.nolagg.spawnlimit.SpawnHandler;

public class NLListener implements Listener {
	
	public static long loadedChunks = 0;
	public static long unloadedChunks = 0;
	public static long genChunks = 0;
			
	@EventHandler(priority = EventPriority.HIGHEST)
	public void onBlockPhysics(BlockPhysicsEvent event) {
		if (event.isCancelled()) return;
		Material type = event.getBlock().getType();
		if (NoLagg.useSpawnLimits) {
			if (type == Material.GRAVEL || type == Material.SAND) {
				EntityLimiter limiter = SpawnHandler.GENERALHANDLER.getEntityLimits(event.getBlock().getWorld(), "falling" + type.toString());
				if (limiter != null) {
					if (limiter.canSpawn()) {
						limiter.spawn();
					} else {
						event.setCancelled(true);
					}
				}
			}
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onItemSpawn(ItemSpawnEvent event) {
		if (event.isCancelled()) return;
		if (NoLagg.useSpawnLimits) {
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.start();
			}
			if (!SpawnHandler.getEntityLimits(event.getEntity()).canSpawn()) {
				event.setCancelled(true);
			}
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.next();
			}
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onCreatureSpawn(CreatureSpawnEvent event) {
		if (NoLagg.useSpawnLimits) {
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.start();
			}
			if (event.getSpawnReason() == SpawnReason.CUSTOM) {
				SpawnHandler.ignoreSpawn(event.getEntity());
			} else {
				if (event.getSpawnReason() == SpawnReason.SPAWNER) {
					SpawnHandler.mobSpawnerSpawned(event.getEntity());
				}
				if (!SpawnHandler.getEntityLimits(event.getEntity()).canSpawn()) {
					event.setCancelled(true);
				}
			}
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.next();
			}
		}
	}
	
	@EventHandler(priority = EventPriority.MONITOR)
	public void onCreatureSpawnMonitor(CreatureSpawnEvent event) {
		if (NoLagg.useSpawnLimits) {
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.start();
			}
			if (!event.isCancelled()) {
				SpawnHandler.getEntityLimits(event.getEntity()).spawn();
			}
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.next();
			}
		}
	}
	
	@EventHandler(priority = EventPriority.MONITOR)
	public void onItemSpawnMonitor(ItemSpawnEvent event) {
		if (NoLagg.useSpawnLimits) {
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.start();
			}
			if (!event.isCancelled()) {
				SpawnHandler.getEntityLimits(event.getEntity()).spawn();
			}
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.next();
			}
		}
	}
	
	
	@EventHandler(priority = EventPriority.MONITOR)
	public void onEntityDeath(EntityDeathEvent event) {
		if (NoLagg.isMonitorEnabled) {
			PerformanceMonitor.entitySpawnLimit.start();
		}
		SpawnHandler.handleDespawn(event.getEntity());
		if (NoLagg.isMonitorEnabled) {
			PerformanceMonitor.entitySpawnLimit.next();
		}
	}
	
			
	@EventHandler(priority = EventPriority.MONITOR)
	public void onChunkLoad(ChunkLoadEvent event) {
		if (NoLagg.isMonitorEnabled) {
			PerformanceMonitor.chunkUpdate.start();
		}
		ChunkMetaData data = ChunkMetaData.get(event.getChunk());
		data.touch();
		if (NoLagg.isMonitorEnabled) {
			PerformanceMonitor.chunkUpdate.next();
		}
		loadedChunks++;
		if (NoLagg.useSpawnLimits) {
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.start();
			}
			//properly clear entities in this chunk if needed
			for (Entity e : event.getChunk().getEntities()) {
				if (!SpawnHandler.getEntityLimits(e).handleSpawn()) {
					e.remove();
				}
			}
			if (NoLagg.isMonitorEnabled) {
				PerformanceMonitor.entitySpawnLimit.next();
			}
		}
	}
	
	@EventHandler(priority = EventPriority.LOWEST)
	public void onChunkUnload(ChunkUnloadEvent event) {
		if (event.isCancelled()) return;
		if (NoLagg.isMonitorEnabled) {
			PerformanceMonitor.chunkUpdate.start();
		}
		ChunkMetaData data = ChunkMetaData.get(event.getChunk());
		if (!data.isUnloadable(false) || !data.isExpired()) {
			event.setCancelled(true);
		}
		if (NoLagg.isMonitorEnabled) {
			PerformanceMonitor.chunkUpdate.next();
		}
	}
	
	@EventHandler(priority = EventPriority.MONITOR)
	public void onChunkUnloadMonitor(ChunkUnloadEvent event) {
		if (event.isCancelled()) {
			ChunkMetaData.get(event.getChunk()).markNotUnloadable();
		} else {
			WorldMetaData.get(event.getWorld()).removeChunk(event.getChunk().getX(), event.getChunk().getZ());
			NLListener.unloadedChunks++;
			if (NoLagg.useSpawnLimits) {
				if (NoLagg.isMonitorEnabled) {
					PerformanceMonitor.entitySpawnLimit.start();
				}
				//properly despawn entities
				for (org.bukkit.entity.Entity e : event.getChunk().getEntities()) {
					SpawnHandler.handleDespawn(e);
				}
				if (NoLagg.isMonitorEnabled) {
					PerformanceMonitor.entitySpawnLimit.next();
				}
			}
		}
	}
		
	@EventHandler(priority = EventPriority.MONITOR)
	public void onWorldLoad(WorldLoadEvent event) {
		WorldMetaData.get(event.getWorld());
		AutoSaveChanger.change(event.getWorld());
	}
	
	@EventHandler(priority = EventPriority.MONITOR)
	public void onWorldUnload(WorldUnloadEvent event) {
		WorldMetaData.remove(event.getWorld());
	}
			
	public boolean isTransparent(Material type) {
		return isTransparent(type.getId());
	}
	public boolean isTransparent(int typeid) {
		return !net.minecraft.server.Block.n[typeid];
	}
	public boolean isSemiTransparent(Material mat) {
		switch (mat) {
		case ICE : return true;
		case LEAVES : return true;
		default: return false;
		}
	}
	
}
